# coding: utf-8

# flake8: noqa

"""
    Cohesive Networks SDK

    Cohesive Networks SDK is a thin wrapper around our product APIs providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

from cohesivenet.log_util import Logger

import os
import urllib3
import six

COHESIVE_LOG_LEVEL = os.environ.get("COHESIVE_LOG_LEVEL", "").lower()
Logger.set_null()
if COHESIVE_LOG_LEVEL:
    Logger.set_stream_handler(COHESIVE_LOG_LEVEL)

from cohesivenet.version import VERSION

__version__ = VERSION

from cohesivenet.models.webhook_custom_property import WebhookCustomProperty
from cohesivenet.models.webhook_detail_response import WebhookDetailResponse
from cohesivenet.models.webhook_header import WebhookHeader
from cohesivenet.models.webhook_integration import WebhookIntegration
from cohesivenet.models.webhook_parameter import WebhookParameter
from cohesivenet.models.webhooks_list_response import WebhooksListResponse
from cohesivenet.models.vns3_controller_peer import VNS3ControllerPeer
from cohesivenet.models.vns3_controller import VNS3Controller
from cohesivenet.models.upgrade_license_response_response import (
    UpgradeLicenseResponseResponse,
)
from cohesivenet.models.upgrade_license_response import UpgradeLicenseResponse
from cohesivenet.models.update_server_ssl_request import UpdateServerSSLRequest
from cohesivenet.models.update_remote_support_config_request import (
    UpdateRemoteSupportConfigRequest,
)
from cohesivenet.models.update_peer_request import UpdatePeerRequest
from cohesivenet.models.update_password_request import UpdatePasswordRequest
from cohesivenet.models.update_license_parameters_request import (
    UpdateLicenseParametersRequest,
)
from cohesivenet.models.update_keyset_request import UpdateKeysetRequest
from cohesivenet.models.update_ipsec_tunnel_request import UpdateIpsecTunnelRequest
from cohesivenet.models.update_ipsec_connection_request import (
    UpdateIpsecConnectionRequest,
)
from cohesivenet.models.update_ipsec_address_request import UpdateIpsecAddressRequest
from cohesivenet.models.update_container_image_request import (
    UpdateContainerImageRequest,
)
from cohesivenet.models.update_container_image_detail_response import (
    UpdateContainerImageDetailResponse,
)
from cohesivenet.models.update_container_image_detail import UpdateContainerImageDetail
from cohesivenet.models.update_configure_container_system_request import (
    UpdateConfigureContainerSystemRequest,
)
from cohesivenet.models.update_config_request import UpdateConfigRequest
from cohesivenet.models.update_config_clientpack_request import (
    UpdateConfigClientpackRequest,
)
from cohesivenet.models.update_clientpacks_status_response import (
    UpdateClientpacksStatusResponse,
)
from cohesivenet.models.update_clientpacks_status import UpdateClientpacksStatus
from cohesivenet.models.update_bgp_peer_connection_request import (
    UpdateBGPPeerConnectionRequest,
)
from cohesivenet.models.update_alert_request import UpdateAlertRequest
from cohesivenet.models.update_admin_ui_settings_request import (
    UpdateAdminUISettingsRequest,
)
from cohesivenet.models.topology import Topology
from cohesivenet.models.task_status_detail import TaskStatusDetail
from cohesivenet.models.task_status import TaskStatus
from cohesivenet.models.system_status_detail import SystemStatusDetail
from cohesivenet.models.system_status import SystemStatus
from cohesivenet.models.system_ping_detail import SystemPingDetail
from cohesivenet.models.system_ping import SystemPing
from cohesivenet.models.system_interface_list_response import (
    SystemInterfaceListResponse,
)
from cohesivenet.models.system_interface_detail import SystemInterfaceDetail
from cohesivenet.models.system_interface import SystemInterface
from cohesivenet.models.stop_container_detail_response import (
    StopContainerDetailResponse,
)
from cohesivenet.models.stop_container_detail import StopContainerDetail
from cohesivenet.models.snapshots_list_response import SnapshotsListResponse
from cohesivenet.models.snapshots_list import SnapshotsList
from cohesivenet.models.snapshots_detail_response import SnapshotsDetailResponse
from cohesivenet.models.snapshot_import_status_response import (
    SnapshotImportStatusResponse,
)
from cohesivenet.models.snapshot_import_status import SnapshotImportStatus
from cohesivenet.models.snapshot import Snapshot
from cohesivenet.models.simple_string_response import SimpleStringResponse
from cohesivenet.models.simple_status_response_response import (
    SimpleStatusResponseResponse,
)
from cohesivenet.models.simple_status_response import SimpleStatusResponse
from cohesivenet.models.simple_boolean_response import SimpleBooleanResponse
from cohesivenet.models.server_ssl_detail_response import ServerSSLDetailResponse
from cohesivenet.models.server_ssl_detail import ServerSSLDetail
from cohesivenet.models.runtime_status_detail import RuntimeStatusDetail
from cohesivenet.models.runtime_status import RuntimeStatus
from cohesivenet.models.runtime_config import RuntimeConfig
from cohesivenet.models.running_containers_detail_state import (
    RunningContainersDetailState,
)
from cohesivenet.models.running_containers_detail_response import (
    RunningContainersDetailResponse,
)
from cohesivenet.models.running_containers_detail_network_settings import (
    RunningContainersDetailNetworkSettings,
)
from cohesivenet.models.running_containers_detail import RunningContainersDetail
from cohesivenet.models.run_container_detail_response import RunContainerDetailResponse
from cohesivenet.models.run_container_detail import RunContainerDetail
from cohesivenet.models.routes_list_response import RoutesListResponse
from cohesivenet.models.route import Route
from cohesivenet.models.restart_status_response import RestartStatusResponse
from cohesivenet.models.restart_status import RestartStatus
from cohesivenet.models.restart_request import RestartRequest
from cohesivenet.models.reset_overlay_client_request import ResetOverlayClientRequest
from cohesivenet.models.remote_support_status_response import (
    RemoteSupportStatusResponse,
)
from cohesivenet.models.remote_support_status import RemoteSupportStatus
from cohesivenet.models.reinit_request import ReinitRequest
from cohesivenet.models.reinit_firewall_request import ReinitFirewallRequest
from cohesivenet.models.reboot_request import RebootRequest
from cohesivenet.models.peers_detail_response import PeersDetailResponse
from cohesivenet.models.peers_detail import PeersDetail
from cohesivenet.models.peer_self_request import PeerSelfRequest
from cohesivenet.models.password_reset_response_response import (
    PasswordResetResponseResponse,
)
from cohesivenet.models.password_reset_response import PasswordResetResponse
from cohesivenet.models.overlay_ip_address import OverlayIPAddress
from cohesivenet.models.overlay_clients_list_response import OverlayClientsListResponse
from cohesivenet.models.overlay_client import OverlayClient
from cohesivenet.models.link_history_detail import LinkHistoryDetail
from cohesivenet.models.link_history import LinkHistory
from cohesivenet.models.link_event import LinkEvent
from cohesivenet.models.license_parameters_state import LicenseParametersState
from cohesivenet.models.license_parameters_detail import LicenseParametersDetail
from cohesivenet.models.license_parameters import LicenseParameters
from cohesivenet.models.license_initial import LicenseInitial
from cohesivenet.models.license_detail import LicenseDetail
from cohesivenet.models.license_container_details import LicenseContainerDetails
from cohesivenet.models.license import License
from cohesivenet.models.keyset_status import KeysetStatus
from cohesivenet.models.keyset_detail import KeysetDetail
from cohesivenet.models.ipsec_tunnel_params import IpsecTunnelParams
from cohesivenet.models.ipsec_tunnel_list_response_key_value import (
    IpsecTunnelListResponseKeyValue,
)
from cohesivenet.models.ipsec_tunnel_detail import IpsecTunnelDetail
from cohesivenet.models.ipsec_tunnel import IpsecTunnel
from cohesivenet.models.ipsec_system_detail_response import IpsecSystemDetailResponse
from cohesivenet.models.ipsec_system_detail import IpsecSystemDetail
from cohesivenet.models.ipsec_remote_endpoint_detail import IpsecRemoteEndpointDetail
from cohesivenet.models.ipsec_remote_endpoint import IpsecRemoteEndpoint
from cohesivenet.models.ipsec_local_endpoint import IpsecLocalEndpoint
from cohesivenet.models.interface_action_request import InterfaceActionRequest
from cohesivenet.models.init_license_detail import InitLicenseDetail
from cohesivenet.models.init_ha_request import InitHaRequest
from cohesivenet.models.haid import Haid
from cohesivenet.models.ha_uuid import HaUUID
from cohesivenet.models.ha_sync_status_response import HaSyncStatusResponse
from cohesivenet.models.ha_sync_status import HaSyncStatus
from cohesivenet.models.ha_detail import HaDetail
from cohesivenet.models.ha_config import HaConfig
from cohesivenet.models.gre_endpoint_list_response import GREEndpointListResponse
from cohesivenet.models.gre_endpoint_detail import GREEndpointDetail
from cohesivenet.models.firewall_subgroup_list_response import (
    FirewallSubgroupListResponse,
)
from cohesivenet.models.firewall_rule_operation_response import (
    FirewallRuleOperationResponse,
)
from cohesivenet.models.firewall_rule_operation_data import FirewallRuleOperationData
from cohesivenet.models.firewall_rule_list_response import FirewallRuleListResponse
from cohesivenet.models.firewall_fw_set_list_response import FirewallFWSetListResponse
from cohesivenet.models.firewall_fw_set import FirewallFWSet
from cohesivenet.models.expire_request import ExpireRequest
from cohesivenet.models.error_detail import ErrorDetail
from cohesivenet.models.error_response import ErrorResponse
from cohesivenet.models.disconnet_client_request import DisconnetClientRequest
from cohesivenet.models.delete_firewall_rule_request import DeleteFirewallRuleRequest
from cohesivenet.models.delete_container_image_detail_response import (
    DeleteContainerImageDetailResponse,
)
from cohesivenet.models.delete_container_image_detail import DeleteContainerImageDetail
from cohesivenet.models.delete_container_detail_response import (
    DeleteContainerDetailResponse,
)
from cohesivenet.models.delete_container_detail import DeleteContainerDetail
from cohesivenet.models.create_snapshot_request import CreateSnapshotRequest
from cohesivenet.models.create_route_request import CreateRouteRequest
from cohesivenet.models.create_peer_request import CreatePeerRequest
from cohesivenet.models.create_ipsec_tunnel_request import CreateIpsecTunnelRequest
from cohesivenet.models.create_ipsec_endpoint_request import CreateIpsecEndpointRequest
from cohesivenet.models.create_image_detail_response import CreateImageDetailResponse
from cohesivenet.models.create_image_detail import CreateImageDetail
from cohesivenet.models.create_firewall_rule_request import CreateFirewallRuleRequest
from cohesivenet.models.create_fw_subgroup_request import CreateFWSubgroupRequest
from cohesivenet.models.create_container_image_response import (
    CreateContainerImageResponse,
)
from cohesivenet.models.create_container_image_detail import CreateContainerImageDetail
from cohesivenet.models.create_clientpack_tag_request import CreateClientpackTagRequest
from cohesivenet.models.create_bgp_peer_request import CreateBGPPeerRequest
from cohesivenet.models.create_alert_request import CreateAlertRequest
from cohesivenet.models.create_access_url_request import CreateAccessURLRequest
from cohesivenet.models.create_api_token_request import CreateAPITokenRequest
from cohesivenet.models.container_system_status_detail_response import (
    ContainerSystemStatusDetailResponse,
)
from cohesivenet.models.container_system_status import ContainerSystemStatus
from cohesivenet.models.container_system_ip_list_response import (
    ContainerSystemIPListResponse,
)
from cohesivenet.models.container_system_ip_list import ContainerSystemIPList
from cohesivenet.models.container_system_ip import ContainerSystemIP
from cohesivenet.models.container_system_action_request import (
    ContainerSystemActionRequest,
)
from cohesivenet.models.container_logs_response import ContainerLogsResponse
from cohesivenet.models.container_logs import ContainerLogs
from cohesivenet.models.container_image_list_response import ContainerImageListResponse
from cohesivenet.models.container_image_list import ContainerImageList
from cohesivenet.models.container_image import ContainerImage
from cohesivenet.models.container import Container
from cohesivenet.models.connected_subnets_detail_response import (
    ConnectedSubnetsDetailResponse,
)
from cohesivenet.models.configure_peer_request import ConfigurePeerRequest
from cohesivenet.models.config_detail import ConfigDetail
from cohesivenet.models.commit_container_request import CommitContainerRequest
from cohesivenet.models.cloud_info_detail import CloudInfoDetail
from cohesivenet.models.cloud_info import CloudInfo
from cohesivenet.models.clientpack_tags_response import ClientpackTagsResponse
from cohesivenet.models.clientpack_tags import ClientpackTags
from cohesivenet.models.clientpack_tag_key_request import ClientpackTagKeyRequest
from cohesivenet.models.clientpack_status_response import ClientpackStatusResponse
from cohesivenet.models.clientpack_status import ClientpackStatus
from cohesivenet.models.clientpack_list_response import ClientpackListResponse
from cohesivenet.models.clientpack_detail_response import ClientpackDetailResponse
from cohesivenet.models.clientpack_detail import ClientpackDetail
from cohesivenet.models.client_pack import ClientPack
from cohesivenet.models.calculate_next_clientpack_request import (
    CalculateNextClientpackRequest,
)
from cohesivenet.models.bulk_client_reset_status_response import (
    BulkClientResetStatusResponse,
)
from cohesivenet.models.bulk_client_reset_status import BulkClientResetStatus
from cohesivenet.models.bgp_peer import BGPPeer
from cohesivenet.models.alerts_list_response import AlertsListResponse
from cohesivenet.models.alert_detail_response import AlertDetailResponse
from cohesivenet.models.alert import Alert
from cohesivenet.models.admin_ui_settings_detail_response import (
    AdminUISettingsDetailResponse,
)
from cohesivenet.models.admin_ui_settings_detail import AdminUISettingsDetail
from cohesivenet.models.add_clientpack_request import AddClientpackRequest
from cohesivenet.models.activate_ha_request import ActivateHaRequest
from cohesivenet.models.access_url_list_response import AccessUrlListResponse
from cohesivenet.models.access_url_detail import AccessUrlDetail
from cohesivenet.models.access_url import AccessUrl
from cohesivenet.models.access_token_list_response import AccessTokenListResponse
from cohesivenet.models.access_token_detail import AccessTokenDetail
from cohesivenet.models.access_token import AccessToken
from cohesivenet.rest import HTTPStatus
from cohesivenet.exceptions import UrlLib3ConnExceptions
from cohesivenet.exceptions import CohesiveSDKException
from cohesivenet.exceptions import ApiException
from cohesivenet.exceptions import ApiKeyError
from cohesivenet.exceptions import ApiValueError
from cohesivenet.exceptions import ApiTypeError
from cohesivenet.exceptions import OpenApiException
from cohesivenet.configuration import Configuration
from cohesivenet.vns3_client import VNS3Client
from cohesivenet.api.vns3.system_administration_api import SystemAdministrationApi
from cohesivenet.api.vns3.snapshots_api import SnapshotsApi
from cohesivenet.api.vns3.routing_api import RoutingApi
from cohesivenet.api.vns3.peering_api import PeeringApi
from cohesivenet.api.vns3.overlay_network_api import OverlayNetworkApi
from cohesivenet.api.vns3.network_edge_plugins_api import NetworkEdgePluginsApi
from cohesivenet.api.vns3.monitoring_alerting_api import MonitoringAlertingApi
from cohesivenet.api.vns3.licensing_api import LicensingApi
from cohesivenet.api.vns3.interfaces_api import InterfacesApi
from cohesivenet.api.vns3.ipsec_api import IPsecApi
from cohesivenet.api.vns3.high_availability_api import HighAvailabilityApi
from cohesivenet.api.vns3.firewall_api import FirewallApi
from cohesivenet.api.vns3.configuration_api import ConfigurationApi
from cohesivenet.api.vns3.bgp_api import BGPApi



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from cohesivenet.exceptions import ApiTypeError, ApiValueError


class OverlayNetworkApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            from cohesivenet.vns3_client import VNS3Client

            api_client = VNS3Client()
        self.api_client = api_client

    def delete_clientpack_tag(
        self, clientpack_name, clientpack_tag_key_request, **kwargs
    ):  # noqa: E501
        """delete_clientpack_tag  # noqa: E501

        For deleting individual clientpack tags  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_clientpack_tag(clientpack_name, clientpack_tag_key_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str clientpack_name: name of clientpack (required)
        :param ClientpackTagKeyRequest clientpack_tag_key_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.delete_clientpack_tag_with_http_info(
            clientpack_name, clientpack_tag_key_request, **kwargs
        )  # noqa: E501

    def delete_clientpack_tag_with_http_info(
        self, clientpack_name, clientpack_tag_key_request, **kwargs
    ):  # noqa: E501
        """delete_clientpack_tag  # noqa: E501

        For deleting individual clientpack tags  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_clientpack_tag_with_http_info(clientpack_name, clientpack_tag_key_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str clientpack_name: name of clientpack (required)
        :param ClientpackTagKeyRequest clientpack_tag_key_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["clientpack_name", "clientpack_tag_key_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_clientpack_tag" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'clientpack_name' is set
        if (
            "clientpack_name" not in local_var_params
            or local_var_params["clientpack_name"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `clientpack_name` when calling `delete_clientpack_tag`"
            )  # noqa: E501
        # verify the required parameter 'clientpack_tag_key_request' is set
        if (
            "clientpack_tag_key_request" not in local_var_params
            or local_var_params["clientpack_tag_key_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `clientpack_tag_key_request` when calling `delete_clientpack_tag`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "clientpack_name" in local_var_params:
            path_params["clientpack_name"] = local_var_params[
                "clientpack_name"
            ]  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "clientpack_tag_key_request" in local_var_params:
            body_params = local_var_params["clientpack_tag_key_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpack/{name}",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_clientpack(self, clientpack_name, **kwargs):  # noqa: E501
        """get_clientpack  # noqa: E501

        Returns detailed information about all of the clientpacks in the topology.  If manager's are properly peered, this information can come from any of the controllers.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_clientpack(clientpack_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str clientpack_name: name of clientpack (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: ClientpackDetailResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.get_clientpack_with_http_info(
            clientpack_name, **kwargs
        )  # noqa: E501

    def get_clientpack_with_http_info(self, clientpack_name, **kwargs):  # noqa: E501
        """get_clientpack  # noqa: E501

        Returns detailed information about all of the clientpacks in the topology.  If manager's are properly peered, this information can come from any of the controllers.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_clientpack_with_http_info(clientpack_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str clientpack_name: name of clientpack (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(ClientpackDetailResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["clientpack_name"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_clientpack" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'clientpack_name' is set
        if (
            "clientpack_name" not in local_var_params
            or local_var_params["clientpack_name"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `clientpack_name` when calling `get_clientpack`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "clientpack_name" in local_var_params:
            path_params["clientpack_name"] = local_var_params[
                "clientpack_name"
            ]  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpacks/{name}",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="ClientpackDetailResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_clientpacks(self, **kwargs):  # noqa: E501
        """get_clientpacks  # noqa: E501

        Returns detailed information about all of the clientpacks in the topology. If manager's are properly peered, this information can come from any of the controllers.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_clientpacks(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param bool sorted: Sort resources
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: ClientpackListResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.get_clientpacks_with_http_info(**kwargs)  # noqa: E501

    def get_clientpacks_with_http_info(self, **kwargs):  # noqa: E501
        """get_clientpacks  # noqa: E501

        Returns detailed information about all of the clientpacks in the topology. If manager's are properly peered, this information can come from any of the controllers.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_clientpacks_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param bool sorted: Sort resources
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(ClientpackListResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["sorted"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_clientpacks" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "sorted" in local_var_params:
            query_params.append(("sorted", local_var_params["sorted"]))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpacks",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="ClientpackListResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_clients_status(self, **kwargs):  # noqa: E501
        """get_clients_status  # noqa: E501

        Describe overlay clients  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_clients_status(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: OverlayClientsListResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.get_clients_status_with_http_info(**kwargs)  # noqa: E501

    def get_clients_status_with_http_info(self, **kwargs):  # noqa: E501
        """get_clients_status  # noqa: E501

        Describe overlay clients  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_clients_status_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(OverlayClientsListResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = []  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_clients_status" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/status/clients",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="OverlayClientsListResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_connected_subnets(self, **kwargs):  # noqa: E501
        """get_connected_subnets  # noqa: E501

        Provides information about any connected subnets.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_connected_subnets(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param bool extended_output: Receive verbose information about resources
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: ConnectedSubnetsDetailResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.get_connected_subnets_with_http_info(**kwargs)  # noqa: E501

    def get_connected_subnets_with_http_info(self, **kwargs):  # noqa: E501
        """get_connected_subnets  # noqa: E501

        Provides information about any connected subnets.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_connected_subnets_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param bool extended_output: Receive verbose information about resources
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(ConnectedSubnetsDetailResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["extended_output"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_connected_subnets" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "extended_output" in local_var_params:
            query_params.append(
                ("extended_output", local_var_params["extended_output"])
            )  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/status/connected_subnets",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="ConnectedSubnetsDetailResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_download_clientpack(self, **kwargs):  # noqa: E501
        """get_download_clientpack  # noqa: E501

        Returns clientpack file. Clientpacks are files with the necessary information and credentials  for an overlay client to be connected to the VNS3 topology   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_download_clientpack(name="100.171.10.1", "fileformat"="conf", async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param name str: name of clientpack
        :param fileformat str: format for file (ovpn, conf or tarball)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.get_download_clientpack_with_http_info(**kwargs)  # noqa: E501

    def get_download_clientpack_with_http_info(self, **kwargs):  # noqa: E501
        """get_download_clientpack  # noqa: E501

        Returns clientpack file. Clientpacks are files with the necessary information and credentials  for an overlay client to be connected to the VNS3 topology   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_download_clientpack(name="100.171.10.1", "fileformat"="conf", async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param name str: name of clientpack
        :param fileformat str: format for file (ovpn, conf or tarball)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(file, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["name", "fileformat"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")
        response_type = kwargs.pop("response_type", None)

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_firewall_rule_subgroups" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []

        missing_name = (
            "name" not in local_var_params or local_var_params["name"] is None
        )
        missing_fileformat = (
            "fileformat" not in local_var_params
            or local_var_params["fileformat"] is None
        )
        if missing_fileformat or missing_name:
            raise ApiValueError(
                "Missing the required parameter(s): `name` and `fileformat` are required."
            )

        if "name" in local_var_params:
            query_params.append(("name", local_var_params["name"]))  # noqa: E501
        if "fileformat" in local_var_params:
            query_params.append(
                ("fileformat", local_var_params["fileformat"])
            )  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["text/plain", "application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpack",
            "GET",
            path_params,
            query_params,
            header_params,
            body={},
            post_params=form_params,
            files=local_var_files,
            response_type=response_type
            or "file:%s" % local_var_params["fileformat"],  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def post_calc_next_clientpack(self, **kwargs):  # noqa: E501
        """post_calc_next_clientpack  # noqa: E501

        Get next sequential client pack. Provides sufficient information to call GET /clientpack.  Note, Using this resource against multiple controllers in the same topology could cause distribution of the  same clientpack to multiple overlay devices which is not allowed.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_calc_next_clientpack(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param CalculateNextClientpackRequest calculate_next_clientpack_request:
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.post_calc_next_clientpack_with_http_info(**kwargs)  # noqa: E501

    def post_calc_next_clientpack_with_http_info(self, **kwargs):  # noqa: E501
        """post_calc_next_clientpack  # noqa: E501

        Get next sequential client pack. Provides sufficient information to call GET /clientpack.  Note, Using this resource against multiple controllers in the same topology could cause distribution of the  same clientpack to multiple overlay devices which is not allowed.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_calc_next_clientpack_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param CalculateNextClientpackRequest calculate_next_clientpack_request:
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["calculate_next_clientpack_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_calc_next_clientpack" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "calculate_next_clientpack_request" in local_var_params:
            body_params = local_var_params["calculate_next_clientpack_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpacks/next_available",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def post_clientpack_tag(
        self, clientpack_name, create_clientpack_tag_request, **kwargs
    ):  # noqa: E501
        """post_clientpack_tag  # noqa: E501

        For tagging individual clientpacks.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_clientpack_tag(clientpack_name, create_clientpack_tag_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str clientpack_name: name of clientpack (required)
        :param CreateClientpackTagRequest create_clientpack_tag_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: ClientpackTagsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.post_clientpack_tag_with_http_info(
            clientpack_name, create_clientpack_tag_request, **kwargs
        )  # noqa: E501

    def post_clientpack_tag_with_http_info(
        self, clientpack_name, create_clientpack_tag_request, **kwargs
    ):  # noqa: E501
        """post_clientpack_tag  # noqa: E501

        For tagging individual clientpacks.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_clientpack_tag_with_http_info(clientpack_name, create_clientpack_tag_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str clientpack_name: name of clientpack (required)
        :param CreateClientpackTagRequest create_clientpack_tag_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(ClientpackTagsResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["clientpack_name", "create_clientpack_tag_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_clientpack_tag" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'clientpack_name' is set
        if (
            "clientpack_name" not in local_var_params
            or local_var_params["clientpack_name"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `clientpack_name` when calling `post_clientpack_tag`"
            )  # noqa: E501
        # verify the required parameter 'create_clientpack_tag_request' is set
        if (
            "create_clientpack_tag_request" not in local_var_params
            or local_var_params["create_clientpack_tag_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `create_clientpack_tag_request` when calling `post_clientpack_tag`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "clientpack_name" in local_var_params:
            path_params["clientpack_name"] = local_var_params[
                "clientpack_name"
            ]  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "create_clientpack_tag_request" in local_var_params:
            body_params = local_var_params["create_clientpack_tag_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpack/{name}",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="ClientpackTagsResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def post_reset_all_clients(self, **kwargs):  # noqa: E501
        """post_reset_all_clients  # noqa: E501

        For resetting all of the connections of clients connected to the VNS3 Controller  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_reset_all_clients(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: BulkClientResetStatusResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.post_reset_all_clients_with_http_info(**kwargs)  # noqa: E501

    def post_reset_all_clients_with_http_info(self, **kwargs):  # noqa: E501
        """post_reset_all_clients  # noqa: E501

        For resetting all of the connections of clients connected to the VNS3 Controller  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_reset_all_clients_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(BulkClientResetStatusResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = []  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_reset_all_clients" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clients/reset_all",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="BulkClientResetStatusResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def post_reset_client(self, reset_overlay_client_request, **kwargs):  # noqa: E501
        """post_reset_client  # noqa: E501

        For resetting the connection of a client to a VNS3 Controller  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_reset_client(reset_overlay_client_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param ResetOverlayClientRequest reset_overlay_client_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: ClientpackStatusResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.post_reset_client_with_http_info(
            reset_overlay_client_request, **kwargs
        )  # noqa: E501

    def post_reset_client_with_http_info(
        self, reset_overlay_client_request, **kwargs
    ):  # noqa: E501
        """post_reset_client  # noqa: E501

        For resetting the connection of a client to a VNS3 Controller  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_reset_client_with_http_info(reset_overlay_client_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param ResetOverlayClientRequest reset_overlay_client_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(ClientpackStatusResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["reset_overlay_client_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_reset_client" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'reset_overlay_client_request' is set
        if (
            "reset_overlay_client_request" not in local_var_params
            or local_var_params["reset_overlay_client_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `reset_overlay_client_request` when calling `post_reset_client`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "reset_overlay_client_request" in local_var_params:
            body_params = local_var_params["reset_overlay_client_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/client/reset",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="ClientpackStatusResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def put_add_clientpacks(self, add_clientpack_request, **kwargs):  # noqa: E501
        """put_add_clientpacks  # noqa: E501

        Incrementally add new clientpacks for use  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_add_clientpacks(add_clientpack_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param AddClientpackRequest add_clientpack_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.put_add_clientpacks_with_http_info(
            add_clientpack_request, **kwargs
        )  # noqa: E501

    def put_add_clientpacks_with_http_info(
        self, add_clientpack_request, **kwargs
    ):  # noqa: E501
        """put_add_clientpacks  # noqa: E501

        Incrementally add new clientpacks for use  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_add_clientpacks_with_http_info(add_clientpack_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param AddClientpackRequest add_clientpack_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["add_clientpack_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method put_add_clientpacks" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'add_clientpack_request' is set
        if (
            "add_clientpack_request" not in local_var_params
            or local_var_params["add_clientpack_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `add_clientpack_request` when calling `put_add_clientpacks`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "add_clientpack_request" in local_var_params:
            body_params = local_var_params["add_clientpack_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpacks/add_clientpacks",
            "PUT",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def put_clientpack(self, update_clientpack_request, **kwargs):  # noqa: E501
        """put_clientpack  # noqa: E501

        Change properties of clientpacks; enabling or disabling, checking in or out, or regenerating  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_clientpack(update_clientpack_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param UpdateClientpackRequest update_clientpack_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: OneOfobjectobject
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.put_clientpack_with_http_info(
            update_clientpack_request, **kwargs
        )  # noqa: E501

    def put_clientpack_with_http_info(
        self, update_clientpack_request, **kwargs
    ):  # noqa: E501
        """put_clientpack  # noqa: E501

        Change properties of clientpacks; enabling or disabling, checking in or out, or regenerating  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_clientpack_with_http_info(update_clientpack_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param UpdateClientpackRequest update_clientpack_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(OneOfobjectobject, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["update_clientpack_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method put_clientpack" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'update_clientpack_request' is set
        if (
            "update_clientpack_request" not in local_var_params
            or local_var_params["update_clientpack_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `update_clientpack_request` when calling `put_clientpack`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "update_clientpack_request" in local_var_params:
            body_params = local_var_params["update_clientpack_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpack",
            "PUT",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="OneOfobjectobject",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def put_disconnect_clientpack(
        self, clientpack_name, disconnet_client_request, **kwargs
    ):  # noqa: E501
        """put_disconnect_clientpack  # noqa: E501

        Force disconnect client for named clientpack  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_disconnect_clientpack(clientpack_name, disconnet_client_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str clientpack_name: name of clientpack (required)
        :param DisconnetClientRequest disconnet_client_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.put_disconnect_clientpack_with_http_info(
            clientpack_name, disconnet_client_request, **kwargs
        )  # noqa: E501

    def put_disconnect_clientpack_with_http_info(
        self, clientpack_name, disconnet_client_request, **kwargs
    ):  # noqa: E501
        """put_disconnect_clientpack  # noqa: E501

        Force disconnect client for named clientpack  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_disconnect_clientpack_with_http_info(clientpack_name, disconnet_client_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str clientpack_name: name of clientpack (required)
        :param DisconnetClientRequest disconnet_client_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["clientpack_name", "disconnet_client_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method put_disconnect_clientpack" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'clientpack_name' is set
        if (
            "clientpack_name" not in local_var_params
            or local_var_params["clientpack_name"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `clientpack_name` when calling `put_disconnect_clientpack`"
            )  # noqa: E501
        # verify the required parameter 'disconnet_client_request' is set
        if (
            "disconnet_client_request" not in local_var_params
            or local_var_params["disconnet_client_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `disconnet_client_request` when calling `put_disconnect_clientpack`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "clientpack_name" in local_var_params:
            path_params["clientpack_name"] = local_var_params[
                "clientpack_name"
            ]  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "disconnet_client_request" in local_var_params:
            body_params = local_var_params["disconnet_client_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpack/{name}",
            "PUT",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def put_update_clientpacks(
        self, update_config_clientpack_request, **kwargs
    ):  # noqa: E501
        """put_update_clientpacks  # noqa: E501

        For bulk set of the enabled (true/false) state for all clientpacks and the checked_out (true/false) state for all clientpacks.  This enables a variety of work flows by calling these functions after key generation,  but before general provisioning of addresses to devivces   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_update_clientpacks(update_config_clientpack_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param UpdateConfigClientpackRequest update_config_clientpack_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: UpdateClientpacksStatusResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.put_update_clientpacks_with_http_info(
            update_config_clientpack_request, **kwargs
        )  # noqa: E501

    def put_update_clientpacks_with_http_info(
        self, update_config_clientpack_request, **kwargs
    ):  # noqa: E501
        """put_update_clientpacks  # noqa: E501

        For bulk set of the enabled (true/false) state for all clientpacks and the checked_out (true/false) state for all clientpacks.  This enables a variety of work flows by calling these functions after key generation,  but before general provisioning of addresses to devivces   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_update_clientpacks_with_http_info(update_config_clientpack_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param UpdateConfigClientpackRequest update_config_clientpack_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(UpdateClientpacksStatusResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["update_config_clientpack_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method put_update_clientpacks" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'update_config_clientpack_request' is set
        if (
            "update_config_clientpack_request" not in local_var_params
            or local_var_params["update_config_clientpack_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `update_config_clientpack_request` when calling `put_update_clientpacks`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "update_config_clientpack_request" in local_var_params:
            body_params = local_var_params["update_config_clientpack_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/clientpacks",
            "PUT",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="UpdateClientpacksStatusResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class LicenseInitial(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "capabilities": "list[str]",
        "finalized": "bool",
        "license": "str",
        "license_present": "bool",
        "default_topology": "Topology",
    }

    attribute_map = {
        "capabilities": "capabilities",
        "finalized": "finalized",
        "license": "license",
        "license_present": "license_present",
        "default_topology": "default_topology",
    }

    def __init__(
        self,
        capabilities=None,
        finalized=None,
        license=None,
        license_present=None,
        default_topology=None,
    ):  # noqa: E501
        """LicenseInitial - a model defined in OpenAPI"""  # noqa: E501

        self._capabilities = None
        self._finalized = None
        self._license = None
        self._license_present = None
        self._default_topology = None
        self.discriminator = None

        if capabilities is not None:
            self.capabilities = capabilities
        if finalized is not None:
            self.finalized = finalized
        if license is not None:
            self.license = license
        if license_present is not None:
            self.license_present = license_present
        if default_topology is not None:
            self.default_topology = default_topology

    @property
    def capabilities(self):
        """Gets the capabilities of this LicenseInitial.  # noqa: E501

        Features available such as eBGP, CloudWAN, Containers etc.  # noqa: E501

        :return: The capabilities of this LicenseInitial.  # noqa: E501
        :rtype: list[str]
        """
        return self._capabilities

    @capabilities.setter
    def capabilities(self, capabilities):
        """Sets the capabilities of this LicenseInitial.

        Features available such as eBGP, CloudWAN, Containers etc.  # noqa: E501

        :param capabilities: The capabilities of this LicenseInitial.  # noqa: E501
        :type: list[str]
        """

        self._capabilities = capabilities

    @property
    def finalized(self):
        """Gets the finalized of this LicenseInitial.  # noqa: E501


        :return: The finalized of this LicenseInitial.  # noqa: E501
        :rtype: bool
        """
        return self._finalized

    @finalized.setter
    def finalized(self, finalized):
        """Sets the finalized of this LicenseInitial.


        :param finalized: The finalized of this LicenseInitial.  # noqa: E501
        :type: bool
        """

        self._finalized = finalized

    @property
    def license(self):
        """Gets the license of this LicenseInitial.  # noqa: E501

        State of license, accepted, in-progress, failed  # noqa: E501

        :return: The license of this LicenseInitial.  # noqa: E501
        :rtype: str
        """
        return self._license

    @license.setter
    def license(self, license):
        """Sets the license of this LicenseInitial.

        State of license, accepted, in-progress, failed  # noqa: E501

        :param license: The license of this LicenseInitial.  # noqa: E501
        :type: str
        """

        self._license = license

    @property
    def license_present(self):
        """Gets the license_present of this LicenseInitial.  # noqa: E501


        :return: The license_present of this LicenseInitial.  # noqa: E501
        :rtype: bool
        """
        return self._license_present

    @license_present.setter
    def license_present(self, license_present):
        """Sets the license_present of this LicenseInitial.


        :param license_present: The license_present of this LicenseInitial.  # noqa: E501
        :type: bool
        """

        self._license_present = license_present

    @property
    def default_topology(self):
        """Gets the default_topology of this LicenseInitial.  # noqa: E501


        :return: The default_topology of this LicenseInitial.  # noqa: E501
        :rtype: Topology
        """
        return self._default_topology

    @default_topology.setter
    def default_topology(self, default_topology):
        """Sets the default_topology of this LicenseInitial.


        :param default_topology: The default_topology of this LicenseInitial.  # noqa: E501
        :type: Topology
        """

        self._default_topology = default_topology

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LicenseInitial):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from functools import partial as bind

from cohesivenet import data_types
from cohesivenet.clouds import networkmath
from cohesivenet.macros import api_operations, state


def create_local_gateway_route(client, local_cidr, **route_kwargs):
    """[summary]

    Arguments:
        client {[type]} -- [description]
        local_cidr {[type]} -- [description]

    Keyword Arguments:
        gateway {[type]} -- [description] (default: {None})

    Returns:
        OperationResult
    """
    route = dict(
        **{
            "cidr": local_cidr,
            "description": "Local Underlay Network Routing",
            "interface": "eth0",
            "gateway": networkmath.get_default_gateway(local_cidr),
            "advertise": "False",
            "metric": 0,
        },
        **route_kwargs
    )
    return api_operations.try_api_call(
        client.routing.post_create_route, route, should_raise=True
    )


def create_route_advertisements(
    clients, local_subnets
) -> data_types.BulkOperationResult:
    """create_route_advertisements Create a route advertisement for controllers network

    Arguments:
        clients {List[VNS3Client]}
        local_subnets {List[str]} - order should correspond with clients list

    Returns:
        data_types.BulkOperationResult
    """
    assert len(clients) == len(
        local_subnets
    ), "clients list length must equal local_subnets list length"

    invalid = []
    for index, client in enumerate(clients):
        private_ip = state.get_primary_private_ip(client)
        if not networkmath.subnet_contains_ipv4(private_ip, local_subnets[index]):
            invalid.append("%s not in %s" % (private_ip, local_subnets[index]))

    if len(invalid):
        raise AssertionError(
            "Invalid subnets provided for clients: %s." % ",".join(invalid)
        )

    def _create_route(_client, subnet):
        return _client.routing.post_create_route(
            {
                "cidr": subnet,
                "description": "Local subnet advertisement",
                "advertise": True,
                "gateway": "",
            }
        )

    bound_api_calls = [
        bind(_create_route, client, local_subnets[index])
        for index, client in enumerate(clients)
    ]

    return api_operations.__bulk_call_api(bound_api_calls)



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class CreateRouteRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "cidr": "str",
        "description": "str",
        "interface": "str",
        "gateway": "str",
        "tunnel": "int",
        "advertise": "bool",
        "metric": "int",
    }

    attribute_map = {
        "cidr": "cidr",
        "description": "description",
        "interface": "interface",
        "gateway": "gateway",
        "tunnel": "tunnel",
        "advertise": "advertise",
        "metric": "metric",
    }

    def __init__(
        self,
        cidr=None,
        description=None,
        interface=None,
        gateway=None,
        tunnel=None,
        advertise=None,
        metric=None,
    ):  # noqa: E501
        """CreateRouteRequest - a model defined in OpenAPI"""  # noqa: E501

        self._cidr = None
        self._description = None
        self._interface = None
        self._gateway = None
        self._tunnel = None
        self._advertise = None
        self._metric = None
        self.discriminator = None

        self.cidr = cidr
        if description is not None:
            self.description = description
        if interface is not None:
            self.interface = interface
        if gateway is not None:
            self.gateway = gateway
        if tunnel is not None:
            self.tunnel = tunnel
        if advertise is not None:
            self.advertise = advertise
        if metric is not None:
            self.metric = metric

    @property
    def cidr(self):
        """Gets the cidr of this CreateRouteRequest.  # noqa: E501

        CIDR of a route that the VNS3 Controller has access  to that it wants to publish throughout the  Routing tables of the overlay network   # noqa: E501

        :return: The cidr of this CreateRouteRequest.  # noqa: E501
        :rtype: str
        """
        return self._cidr

    @cidr.setter
    def cidr(self, cidr):
        """Sets the cidr of this CreateRouteRequest.

        CIDR of a route that the VNS3 Controller has access  to that it wants to publish throughout the  Routing tables of the overlay network   # noqa: E501

        :param cidr: The cidr of this CreateRouteRequest.  # noqa: E501
        :type: str
        """
        if cidr is None:
            raise ValueError(
                "Invalid value for `cidr`, must not be `None`"
            )  # noqa: E501

        self._cidr = cidr

    @property
    def description(self):
        """Gets the description of this CreateRouteRequest.  # noqa: E501


        :return: The description of this CreateRouteRequest.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this CreateRouteRequest.


        :param description: The description of this CreateRouteRequest.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def interface(self):
        """Gets the interface of this CreateRouteRequest.  # noqa: E501

        Sets the interface where this route will be advertised.  # noqa: E501

        :return: The interface of this CreateRouteRequest.  # noqa: E501
        :rtype: str
        """
        return self._interface

    @interface.setter
    def interface(self, interface):
        """Sets the interface of this CreateRouteRequest.

        Sets the interface where this route will be advertised.  # noqa: E501

        :param interface: The interface of this CreateRouteRequest.  # noqa: E501
        :type: str
        """

        self._interface = interface

    @property
    def gateway(self):
        """Gets the gateway of this CreateRouteRequest.  # noqa: E501

        If interface is set, a specific gateway address reachable from that interface  # noqa: E501

        :return: The gateway of this CreateRouteRequest.  # noqa: E501
        :rtype: str
        """
        return self._gateway

    @gateway.setter
    def gateway(self, gateway):
        """Sets the gateway of this CreateRouteRequest.

        If interface is set, a specific gateway address reachable from that interface  # noqa: E501

        :param gateway: The gateway of this CreateRouteRequest.  # noqa: E501
        :type: str
        """

        self._gateway = gateway

    @property
    def tunnel(self):
        """Gets the tunnel of this CreateRouteRequest.  # noqa: E501

        numerical reference for the GRE endpoint id (must provide either tunnel OR interface)  # noqa: E501

        :return: The tunnel of this CreateRouteRequest.  # noqa: E501
        :rtype: int
        """
        return self._tunnel

    @tunnel.setter
    def tunnel(self, tunnel):
        """Sets the tunnel of this CreateRouteRequest.

        numerical reference for the GRE endpoint id (must provide either tunnel OR interface)  # noqa: E501

        :param tunnel: The tunnel of this CreateRouteRequest.  # noqa: E501
        :type: int
        """

        self._tunnel = tunnel

    @property
    def advertise(self):
        """Gets the advertise of this CreateRouteRequest.  # noqa: E501

        advertise route to overlay network  # noqa: E501

        :return: The advertise of this CreateRouteRequest.  # noqa: E501
        :rtype: bool
        """
        return self._advertise

    @advertise.setter
    def advertise(self, advertise):
        """Sets the advertise of this CreateRouteRequest.

        advertise route to overlay network  # noqa: E501

        :param advertise: The advertise of this CreateRouteRequest.  # noqa: E501
        :type: bool
        """

        self._advertise = advertise

    @property
    def metric(self):
        """Gets the metric of this CreateRouteRequest.  # noqa: E501

        weight for route  # noqa: E501

        :return: The metric of this CreateRouteRequest.  # noqa: E501
        :rtype: int
        """
        return self._metric

    @metric.setter
    def metric(self, metric):
        """Sets the metric of this CreateRouteRequest.

        weight for route  # noqa: E501

        :param metric: The metric of this CreateRouteRequest.  # noqa: E501
        :type: int
        """

        self._metric = metric

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateRouteRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from cohesivenet import util, Logger, VNS3Client, CohesiveSDKException


def format_rule(rule, state):
    err_none = None
    if util.is_formattable_string(rule):
        try:
            return rule.format(**state), err_none
        except KeyError as e:
            return (
                rule,
                "Rule format error: missing state args %s" % ",".join(e.args),
            )
    return rule, err_none


def create_firewall_policy(client: VNS3Client, firewall_rules, state={}):
    """Create group of firewall rules

    Arguments:
        client {VNS3Client}
        firewall_rules {List[CreateFirewallRuleRequest]} - [{
            'position': int,
            'rule': str
        }, ...]

    Keyword Arguments:
        state {dict} - State to format rules with. (can call client.controller_state)

    Returns:
        Tuple[List[str], List[str]] - success, errors
    """
    successes = []
    errors = []
    Logger.debug(
        "Creating firewall policy.",
        host=client.host_uri,
        rule_count=len(firewall_rules),
    )
    for i, rule_args in enumerate(firewall_rules):
        rule, err = format_rule(rule_args["rule"], state)
        if err:
            errors.append(err)
            continue

        rule_args.update(rule=rule)
        client.firewall.post_create_firewall_rule(rule_args)
        successes.append(
            'Rule "%s" inserted at position %d' % (rule, rule_args["position"])
        )
    return successes, errors


def __construct_proposed_firewall_list(rule_args_list, state=None):
    _state = state or {}
    errors = []
    new_firewall = []
    for rule_args in rule_args_list:
        rule, err = format_rule(rule_args["rule"], _state)
        if err:
            errors.append(err)
            continue

        position = rule_args.get("position", -1)
        if position == -1:
            new_firewall.append(rule)
        elif type(position) is int:
            new_firewall.insert(position, rule)
        else:
            errors.append("Rule position %s invalid. [RULE='%s']" % (position, rule))

    return new_firewall, errors


def __firewall_resp_to_list(firewall_raw_response):
    return [
        rule_tuple[0].strip()
        for rule_tuple in sorted(
            firewall_raw_response.response, key=lambda r: int(r[1])
        )
    ]


def assert_rule_policy(client: VNS3Client, rules, should_fix=False):
    """Assert rule policy contains expected rules

    Arguments:
        client {VNS3Client}
        rules {List[dict]}

    Keyword Arguments:
        should_fix {bool} - if false, raise Error, else, update firewall

    Raises:
        CohesiveSDKException - raised if invalid firewall rules provided
        AssertionError - raised if should_fix=False and provided rules dont match VNS3

    Returns:
        List[str] - ordered list of firewall rules
    """
    current_firewall = __firewall_resp_to_list(client.firewall.get_firewall_rules())
    new_firewall, errors = __construct_proposed_firewall_list(
        rules, state=client.controller_state
    )
    if errors:
        raise CohesiveSDKException(
            "Invalid firewall rules provided. Errors=%s" % (errors)
        )

    if current_firewall == new_firewall:
        Logger.info("Current firewall is correct. No-op.", host=client.host_uri)
        return current_firewall

    Logger.info(
        "Firewall configuration drift. Expected: %s != %s."
        % (new_firewall, current_firewall),
        host=client.host_uri,
    )

    if not should_fix:
        raise AssertionError(
            "Firewalls did not match for VNS3 @ %s. Current firewall %s != %s."
            % (client.host_uri, current_firewall, new_firewall)
        )

    # operations: insert, delete
    OP_INS = "insert"
    OP_DEL = "delete"
    firewall_edits = []
    for i, rule in enumerate(new_firewall):
        if len(current_firewall) <= i:
            operation = OP_INS
        elif current_firewall[i] == rule:
            continue
        else:
            # current firewall rule is incorrect.
            # now, minimize operations to get correct
            # if can insert OR delete, prefer delete
            # ie. if next rule is the correct rule, del this rule
            operation = (
                OP_DEL
                if len(current_firewall) > i + 1 and current_firewall[i + 1] == rule
                else OP_INS
            )

        firewall_edits.append("%s:%s" % (operation, i))
        if operation == OP_INS:
            client.firewall.post_create_firewall_rule({"rule": rule, "position": i})
            current_firewall.insert(i, rule)
        else:  # operation == OP_DEL:
            client.firewall.delete_firewall_rule_by_position(i)
            del current_firewall[i]

    Logger.debug(
        "%s network operations required to fix firewall: %s"
        % (len(firewall_edits), firewall_edits),
        host=client.host_uri,
    )

    return __firewall_resp_to_list(client.firewall.get_firewall_rules())



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class AdminUISettingsDetailResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "enabled": "bool",
        "admin_username": "str",
        "admin_password": "str",
    }

    attribute_map = {
        "enabled": "enabled",
        "admin_username": "admin_username",
        "admin_password": "admin_password",
    }

    def __init__(
        self, enabled=None, admin_username=None, admin_password=None
    ):  # noqa: E501
        """AdminUISettingsDetailResponse - a model defined in OpenAPI"""  # noqa: E501

        self._enabled = None
        self._admin_username = None
        self._admin_password = None
        self.discriminator = None

        if enabled is not None:
            self.enabled = enabled
        if admin_username is not None:
            self.admin_username = admin_username
        if admin_password is not None:
            self.admin_password = admin_password

    @property
    def enabled(self):
        """Gets the enabled of this AdminUISettingsDetailResponse.  # noqa: E501


        :return: The enabled of this AdminUISettingsDetailResponse.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this AdminUISettingsDetailResponse.


        :param enabled: The enabled of this AdminUISettingsDetailResponse.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    @property
    def admin_username(self):
        """Gets the admin_username of this AdminUISettingsDetailResponse.  # noqa: E501


        :return: The admin_username of this AdminUISettingsDetailResponse.  # noqa: E501
        :rtype: str
        """
        return self._admin_username

    @admin_username.setter
    def admin_username(self, admin_username):
        """Sets the admin_username of this AdminUISettingsDetailResponse.


        :param admin_username: The admin_username of this AdminUISettingsDetailResponse.  # noqa: E501
        :type: str
        """

        self._admin_username = admin_username

    @property
    def admin_password(self):
        """Gets the admin_password of this AdminUISettingsDetailResponse.  # noqa: E501


        :return: The admin_password of this AdminUISettingsDetailResponse.  # noqa: E501
        :rtype: str
        """
        return self._admin_password

    @admin_password.setter
    def admin_password(self, admin_password):
        """Sets the admin_password of this AdminUISettingsDetailResponse.


        :param admin_password: The admin_password of this AdminUISettingsDetailResponse.  # noqa: E501
        :type: str
        """

        self._admin_password = admin_password

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AdminUISettingsDetailResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class CalculateNextClientpackRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"low_ip": "str", "high_ip": "str", "include_disabled": "bool"}

    attribute_map = {
        "low_ip": "low_ip",
        "high_ip": "high_ip",
        "include_disabled": "include_disabled",
    }

    def __init__(self, low_ip=None, high_ip=None, include_disabled=False):  # noqa: E501
        """CalculateNextClientpackRequest - a model defined in OpenAPI"""  # noqa: E501

        self._low_ip = None
        self._high_ip = None
        self._include_disabled = None
        self.discriminator = None

        if low_ip is not None:
            self.low_ip = low_ip
        if high_ip is not None:
            self.high_ip = high_ip
        if include_disabled is not None:
            self.include_disabled = include_disabled

    @property
    def low_ip(self):
        """Gets the low_ip of this CalculateNextClientpackRequest.  # noqa: E501

        Set the lower bound for the resulting IP  # noqa: E501

        :return: The low_ip of this CalculateNextClientpackRequest.  # noqa: E501
        :rtype: str
        """
        return self._low_ip

    @low_ip.setter
    def low_ip(self, low_ip):
        """Sets the low_ip of this CalculateNextClientpackRequest.

        Set the lower bound for the resulting IP  # noqa: E501

        :param low_ip: The low_ip of this CalculateNextClientpackRequest.  # noqa: E501
        :type: str
        """

        self._low_ip = low_ip

    @property
    def high_ip(self):
        """Gets the high_ip of this CalculateNextClientpackRequest.  # noqa: E501

        Set the upper bound for the resulting IP  # noqa: E501

        :return: The high_ip of this CalculateNextClientpackRequest.  # noqa: E501
        :rtype: str
        """
        return self._high_ip

    @high_ip.setter
    def high_ip(self, high_ip):
        """Sets the high_ip of this CalculateNextClientpackRequest.

        Set the upper bound for the resulting IP  # noqa: E501

        :param high_ip: The high_ip of this CalculateNextClientpackRequest.  # noqa: E501
        :type: str
        """

        self._high_ip = high_ip

    @property
    def include_disabled(self):
        """Gets the include_disabled of this CalculateNextClientpackRequest.  # noqa: E501

        Allows clientpack allocation from the disabled pool, for workflows where all clientpacks are disabled at the start.  # noqa: E501

        :return: The include_disabled of this CalculateNextClientpackRequest.  # noqa: E501
        :rtype: bool
        """
        return self._include_disabled

    @include_disabled.setter
    def include_disabled(self, include_disabled):
        """Sets the include_disabled of this CalculateNextClientpackRequest.

        Allows clientpack allocation from the disabled pool, for workflows where all clientpacks are disabled at the start.  # noqa: E501

        :param include_disabled: The include_disabled of this CalculateNextClientpackRequest.  # noqa: E501
        :type: bool
        """

        self._include_disabled = include_disabled

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CalculateNextClientpackRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import asyncio
import functools
import json
import math
import os
import time
import re

from copy import deepcopy
from contextlib import contextmanager
from typing import Dict, Tuple, List, Callable, Union, Awaitable

from cohesivenet.log_util import scrub_sensitive


def force_async(fn):
    """Turns a sync function to async function using threads

    Arguments:
        fn {function}

    Returns:
        function - awaitable function
    """
    from concurrent.futures import ThreadPoolExecutor

    pool = ThreadPoolExecutor()

    @functools.wraps(fn)
    def async_wrapper(*args, **kwargs):
        future = pool.submit(fn, *args, **kwargs)
        return asyncio.wrap_future(future)  # make it awaitable

    return async_wrapper


def run_parallel(*coroutines):
    loop = asyncio.get_event_loop()
    results = loop.run_until_complete(asyncio.gather(*coroutines))
    return results


def run_pipe(init_data, steps: List[Tuple[str, Callable]]):
    data = deepcopy(init_data)
    total_steps = len(steps)
    print(
        "Running pipe [steps=%s] [inputs=%s]"
        % (total_steps, scrub_sensitive(init_data))
    )

    for step_i, (step_name, func) in enumerate(steps):
        step_num = step_i + 1
        print(
            "Running step [step=%s/%s] [name=%s]" % (step_num, total_steps, step_name)
        )
        response = func(data)
        outputs = response.get("outputs", {})
        data.update(outputs)
        print("Step %s/%s finished. Outputs: %s" % (step_num, total_steps, outputs))
    return data


def run_pipe_async(
    init_data, steps: List[Tuple[str, Union[Callable, List[Awaitable]]]]
):
    """Run pipeline of step functions, running in parallel if

    Arguments:
        init_data {[type]} -- [description]
        steps {List[Tuple[str, Union[Callable, List[Awaitable]]]]} -- [description]

    Returns:
        [type] -- [description]
    """
    data = deepcopy(init_data)
    total_steps = len(steps)
    print(
        "Running pipe [steps=%s] [inputs=%s]"
        % (total_steps, scrub_sensitive(init_data))
    )
    for step_i, (step_name, step_func) in enumerate(steps):
        step_num = step_i + 1
        print(
            "Running pipe step [step=%s/%s] [name=%s]"
            % (step_num, total_steps, step_name)
        )
        if type(step_func) is list:
            print("Running async substeps")
            step_responses = run_parallel(*(func(data) for func in step_func))
            print("Substeps finished. Outputs: %s" % step_responses)
            for response in step_responses:
                data.update(response.get("outputs", {}))
        else:
            response = step_func(data)
            outputs = response.get("outputs", {})
            data.update(outputs)
            print("Step %s/%s finished. Outputs: %s" % (step_num, total_steps, outputs))
    return data


def take_keys(keys: List[str], data_dict: Dict):
    """Take keys from dict

    Arguments:
        keys {List[str]} -- Keys it include in output dict
        data_dict {Dict}

    Returns:
        [Dict]
    """
    return {k: v for k, v in data_dict.items() if k in keys}


def flatten_dict(d, prefix=None, joinchar="__"):
    """flatten_dict Flatten nested dictionary, joining paths into single string key

    Arguments:
        d {[type]} -- [description]

    Keyword Arguments:
        prefix {str} -- Prefix string for joining keys (default: {None})
        joinchar {str} -- String for joining nested keys (default: {'__'})

    Returns:
        [Dict] -- Dict of depth 1
    """
    key_value_pairs = {}

    def _prefix(k):
        return k if not prefix else "%s%s%s" % (prefix, joinchar, k)

    for k, v in d.items():
        if type(v) is dict:
            key_value_pairs.update(
                flatten_dict(v, prefix=_prefix(k), joinchar=joinchar)
            )
        else:
            key_value_pairs[_prefix(k)] = v
    return key_value_pairs


def unflatten_dict(d, splitchar="__"):
    """unflatten_dict Build nested dictionary based on keys

    Arguments:
        d {Dict}

    Keyword Arguments:
        splitchar {str} -- str marking nested dictionary (default: {'__'})

    Returns:
        [Dict]
    """
    resp = {}
    for flatkey, v in d.items():
        keyparts = flatkey.split(splitchar)
        _target = resp
        final_key = keyparts[-1]
        for key in keyparts[:-1]:
            if key not in _target:
                _target[key] = {}
            _target = _target[key]
        _target[final_key] = v
    return resp


def map_type(s, expected_type, strict=True):
    _def_return_val = s
    try:
        if type(s) is list:
            return s
        elif "list" in str(expected_type).lower():
            _def_return_val = []
            return json.loads(s)
        elif not s or s.lower() in ("none", ""):
            return None
        elif expected_type is bool:
            return s.lower() == "true"

        return expected_type(s)
    except ValueError:
        if not strict:
            return _def_return_val
        raise


def get_path(data_dict, key_path, fail=False):
    """get_path

    Arguments:
        data_dict {dict} -- [description]
        key_path {str or List of strings} -- a.b.c or ['a', 'b', 'c']

    Keyword Arguments:
        fail {bool} -- Raise exception if does not exist (default: {False})

    Raises:
        Exception: Generic not found exception

    Returns:
        [any] -- value at key path
    """
    _target = data_dict
    steps = key_path if type(key_path) is list else key_path.split(".")
    for step in steps:
        if type(_target) is not dict or step not in _target:
            if fail:
                raise Exception('Path "%s" does not exist' % key_path)
            return None
        _target = _target.get(step)
    return _target


def map_dict_values(func_map, data_dict):
    return {k: func_map.get(k, lambda x: x)(v) for k, v in data_dict.items()}


def map_dict_keypaths(key_map, data_dict):
    updates = {
        new_key: get_path(data_dict, key_path) for key_path, new_key in key_map.items()
    }
    return {**data_dict, **updates}


def is_formattable_string(s):
    regexp = re.compile(r"{.*}")
    return regexp.search(s)


def partition_list_groups(object_list, number_partitions):
    """Partition list of objects into groups

    Arguments:
        object_list {List[Any]}
        number_partitions {int} -- [description]

    Returns:
        [List[List[Any]]]
    """
    if number_partitions <= 1:
        return object_list

    object_count = len(object_list)
    partition_size = math.floor(object_count / number_partitions)
    leftovers = object_count % number_partitions
    leftover_set = object_list[-leftovers:] if leftovers else []
    rounded_list = object_list[:-leftovers] if leftovers else object_list

    return [
        rounded_list[i * partition_size : (i + 1) * partition_size]
        + ([leftover_set[i]] if len(leftover_set) > i else [])
        for i in range(number_partitions)
    ]


def partition_list_ratios(object_list, partition_ratios):
    """Partition list of objects into groups based on ratios list

    Arguments:
        object_list {List[Any]}
        partition_ratios {List[float]}

    Returns:
        Dict[str, List[Any]] - {
            '0.45': [...],
            '0.25': [...],
            '0.30': [...]
        }
    """
    assert math.isclose(sum(partition_ratios), 1.0), "Ratios must sum to 1"

    number_partitions = len(partition_ratios)
    if number_partitions <= 1:
        return object_list

    partition_sizes = [round(r * len(object_list)) for r in partition_ratios]
    _cursor = 0
    partitions = {}
    for i, size in enumerate(partition_sizes):
        partitions[str(partition_ratios[i])] = object_list[_cursor : (_cursor + size)]
        _cursor += size
    return partitions


def random_timestamp_filename(file_type=None):
    timestr = str(time.time()).replace(".", "_")
    if file_type:
        return "%s.%s" % (timestr, file_type)
    return timestr


@contextmanager
def cd(newdir):
    prevdir = os.getcwd()
    os.chdir(os.path.expanduser(newdir))
    try:
        yield
    finally:
        os.chdir(prevdir)



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class Container(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "str",
        "created": "str",
        "path": "str",
        "args": "list[str]",
        "config": "dict(str, object)",
        "state": "RunningContainersDetailState",
        "image": "str",
        "network_settings": "RunningContainersDetailNetworkSettings",
        "resolv_conf_path": "str",
        "hostname_path": "str",
        "hosts_path": "str",
        "name": "str",
        "driver": "str",
        "exec_driver": "str",
        "volumes": "dict(str, object)",
        "volumes_rw": "dict(str, object)",
        "host_config": "dict(str, object)",
    }

    attribute_map = {
        "id": "ID",
        "created": "Created",
        "path": "Path",
        "args": "Args",
        "config": "Config",
        "state": "State",
        "image": "Image",
        "network_settings": "NetworkSettings",
        "resolv_conf_path": "ResolvConfPath",
        "hostname_path": "HostnamePath",
        "hosts_path": "HostsPath",
        "name": "Name",
        "driver": "Driver",
        "exec_driver": "ExecDriver",
        "volumes": "Volumes",
        "volumes_rw": "VolumesRW",
        "host_config": "HostConfig",
    }

    def __init__(
        self,
        id=None,
        created=None,
        path=None,
        args=None,
        config=None,
        state=None,
        image=None,
        network_settings=None,
        resolv_conf_path=None,
        hostname_path=None,
        hosts_path=None,
        name=None,
        driver=None,
        exec_driver=None,
        volumes=None,
        volumes_rw=None,
        host_config=None,
    ):  # noqa: E501
        """Container - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._created = None
        self._path = None
        self._args = None
        self._config = None
        self._state = None
        self._image = None
        self._network_settings = None
        self._resolv_conf_path = None
        self._hostname_path = None
        self._hosts_path = None
        self._name = None
        self._driver = None
        self._exec_driver = None
        self._volumes = None
        self._volumes_rw = None
        self._host_config = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if created is not None:
            self.created = created
        if path is not None:
            self.path = path
        if args is not None:
            self.args = args
        if config is not None:
            self.config = config
        if state is not None:
            self.state = state
        if image is not None:
            self.image = image
        if network_settings is not None:
            self.network_settings = network_settings
        if resolv_conf_path is not None:
            self.resolv_conf_path = resolv_conf_path
        if hostname_path is not None:
            self.hostname_path = hostname_path
        if hosts_path is not None:
            self.hosts_path = hosts_path
        if name is not None:
            self.name = name
        if driver is not None:
            self.driver = driver
        if exec_driver is not None:
            self.exec_driver = exec_driver
        self.volumes = volumes
        self.volumes_rw = volumes_rw
        self.host_config = host_config

    @property
    def id(self):
        """Gets the id of this Container.  # noqa: E501


        :return: The id of this Container.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Container.


        :param id: The id of this Container.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def created(self):
        """Gets the created of this Container.  # noqa: E501


        :return: The created of this Container.  # noqa: E501
        :rtype: str
        """
        return self._created

    @created.setter
    def created(self, created):
        """Sets the created of this Container.


        :param created: The created of this Container.  # noqa: E501
        :type: str
        """

        self._created = created

    @property
    def path(self):
        """Gets the path of this Container.  # noqa: E501


        :return: The path of this Container.  # noqa: E501
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """Sets the path of this Container.


        :param path: The path of this Container.  # noqa: E501
        :type: str
        """

        self._path = path

    @property
    def args(self):
        """Gets the args of this Container.  # noqa: E501


        :return: The args of this Container.  # noqa: E501
        :rtype: list[str]
        """
        return self._args

    @args.setter
    def args(self, args):
        """Sets the args of this Container.


        :param args: The args of this Container.  # noqa: E501
        :type: list[str]
        """

        self._args = args

    @property
    def config(self):
        """Gets the config of this Container.  # noqa: E501


        :return: The config of this Container.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._config

    @config.setter
    def config(self, config):
        """Sets the config of this Container.


        :param config: The config of this Container.  # noqa: E501
        :type: dict(str, object)
        """

        self._config = config

    @property
    def state(self):
        """Gets the state of this Container.  # noqa: E501


        :return: The state of this Container.  # noqa: E501
        :rtype: RunningContainersDetailState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this Container.


        :param state: The state of this Container.  # noqa: E501
        :type: RunningContainersDetailState
        """

        self._state = state

    @property
    def image(self):
        """Gets the image of this Container.  # noqa: E501


        :return: The image of this Container.  # noqa: E501
        :rtype: str
        """
        return self._image

    @image.setter
    def image(self, image):
        """Sets the image of this Container.


        :param image: The image of this Container.  # noqa: E501
        :type: str
        """

        self._image = image

    @property
    def network_settings(self):
        """Gets the network_settings of this Container.  # noqa: E501


        :return: The network_settings of this Container.  # noqa: E501
        :rtype: RunningContainersDetailNetworkSettings
        """
        return self._network_settings

    @network_settings.setter
    def network_settings(self, network_settings):
        """Sets the network_settings of this Container.


        :param network_settings: The network_settings of this Container.  # noqa: E501
        :type: RunningContainersDetailNetworkSettings
        """

        self._network_settings = network_settings

    @property
    def resolv_conf_path(self):
        """Gets the resolv_conf_path of this Container.  # noqa: E501


        :return: The resolv_conf_path of this Container.  # noqa: E501
        :rtype: str
        """
        return self._resolv_conf_path

    @resolv_conf_path.setter
    def resolv_conf_path(self, resolv_conf_path):
        """Sets the resolv_conf_path of this Container.


        :param resolv_conf_path: The resolv_conf_path of this Container.  # noqa: E501
        :type: str
        """

        self._resolv_conf_path = resolv_conf_path

    @property
    def hostname_path(self):
        """Gets the hostname_path of this Container.  # noqa: E501


        :return: The hostname_path of this Container.  # noqa: E501
        :rtype: str
        """
        return self._hostname_path

    @hostname_path.setter
    def hostname_path(self, hostname_path):
        """Sets the hostname_path of this Container.


        :param hostname_path: The hostname_path of this Container.  # noqa: E501
        :type: str
        """

        self._hostname_path = hostname_path

    @property
    def hosts_path(self):
        """Gets the hosts_path of this Container.  # noqa: E501


        :return: The hosts_path of this Container.  # noqa: E501
        :rtype: str
        """
        return self._hosts_path

    @hosts_path.setter
    def hosts_path(self, hosts_path):
        """Sets the hosts_path of this Container.


        :param hosts_path: The hosts_path of this Container.  # noqa: E501
        :type: str
        """

        self._hosts_path = hosts_path

    @property
    def name(self):
        """Gets the name of this Container.  # noqa: E501


        :return: The name of this Container.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Container.


        :param name: The name of this Container.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def driver(self):
        """Gets the driver of this Container.  # noqa: E501


        :return: The driver of this Container.  # noqa: E501
        :rtype: str
        """
        return self._driver

    @driver.setter
    def driver(self, driver):
        """Sets the driver of this Container.


        :param driver: The driver of this Container.  # noqa: E501
        :type: str
        """

        self._driver = driver

    @property
    def exec_driver(self):
        """Gets the exec_driver of this Container.  # noqa: E501


        :return: The exec_driver of this Container.  # noqa: E501
        :rtype: str
        """
        return self._exec_driver

    @exec_driver.setter
    def exec_driver(self, exec_driver):
        """Sets the exec_driver of this Container.


        :param exec_driver: The exec_driver of this Container.  # noqa: E501
        :type: str
        """

        self._exec_driver = exec_driver

    @property
    def volumes(self):
        """Gets the volumes of this Container.  # noqa: E501


        :return: The volumes of this Container.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._volumes

    @volumes.setter
    def volumes(self, volumes):
        """Sets the volumes of this Container.


        :param volumes: The volumes of this Container.  # noqa: E501
        :type: dict(str, object)
        """

        self._volumes = volumes

    @property
    def volumes_rw(self):
        """Gets the volumes_rw of this Container.  # noqa: E501


        :return: The volumes_rw of this Container.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._volumes_rw

    @volumes_rw.setter
    def volumes_rw(self, volumes_rw):
        """Sets the volumes_rw of this Container.


        :param volumes_rw: The volumes_rw of this Container.  # noqa: E501
        :type: dict(str, object)
        """

        self._volumes_rw = volumes_rw

    @property
    def host_config(self):
        """Gets the host_config of this Container.  # noqa: E501


        :return: The host_config of this Container.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._host_config

    @host_config.setter
    def host_config(self, host_config):
        """Sets the host_config of this Container.


        :param host_config: The host_config of this Container.  # noqa: E501
        :type: dict(str, object)
        """

        self._host_config = host_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Container):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class UpdateBGPPeerConnectionRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "ipaddress": "str",
        "asn": "int",
        "access_list": "str",
        "bgp_password": "str",
        "add_network_distance": "bool",
        "add_network_distance_direction": "str",
        "add_network_distance_hops": "int",
    }

    attribute_map = {
        "ipaddress": "ipaddress",
        "asn": "asn",
        "access_list": "access_list",
        "bgp_password": "bgp_password",
        "add_network_distance": "add_network_distance",
        "add_network_distance_direction": "add_network_distance_direction",
        "add_network_distance_hops": "add_network_distance_hops",
    }

    def __init__(
        self,
        ipaddress=None,
        asn=None,
        access_list=None,
        bgp_password=None,
        add_network_distance=None,
        add_network_distance_direction=None,
        add_network_distance_hops=None,
    ):  # noqa: E501
        """UpdateBGPPeerConnectionRequest - a model defined in OpenAPI"""  # noqa: E501

        self._ipaddress = None
        self._asn = None
        self._access_list = None
        self._bgp_password = None
        self._add_network_distance = None
        self._add_network_distance_direction = None
        self._add_network_distance_hops = None
        self.discriminator = None

        self.ipaddress = ipaddress
        self.asn = asn
        if access_list is not None:
            self.access_list = access_list
        if bgp_password is not None:
            self.bgp_password = bgp_password
        if add_network_distance is not None:
            self.add_network_distance = add_network_distance
        if add_network_distance_direction is not None:
            self.add_network_distance_direction = add_network_distance_direction
        if add_network_distance_hops is not None:
            self.add_network_distance_hops = add_network_distance_hops

    @property
    def ipaddress(self):
        """Gets the ipaddress of this UpdateBGPPeerConnectionRequest.  # noqa: E501

        IP address of the desired BGP peer.  # noqa: E501

        :return: The ipaddress of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :rtype: str
        """
        return self._ipaddress

    @ipaddress.setter
    def ipaddress(self, ipaddress):
        """Sets the ipaddress of this UpdateBGPPeerConnectionRequest.

        IP address of the desired BGP peer.  # noqa: E501

        :param ipaddress: The ipaddress of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :type: str
        """
        if ipaddress is None:
            raise ValueError(
                "Invalid value for `ipaddress`, must not be `None`"
            )  # noqa: E501

        self._ipaddress = ipaddress

    @property
    def asn(self):
        """Gets the asn of this UpdateBGPPeerConnectionRequest.  # noqa: E501

        Autonomous system number assigned to device at ipaddress  # noqa: E501

        :return: The asn of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :rtype: int
        """
        return self._asn

    @asn.setter
    def asn(self, asn):
        """Sets the asn of this UpdateBGPPeerConnectionRequest.

        Autonomous system number assigned to device at ipaddress  # noqa: E501

        :param asn: The asn of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :type: int
        """
        if asn is None:
            raise ValueError(
                "Invalid value for `asn`, must not be `None`"
            )  # noqa: E501

        self._asn = asn

    @property
    def access_list(self):
        """Gets the access_list of this UpdateBGPPeerConnectionRequest.  # noqa: E501

        List of \"in permit CIDR\" and/or \"out permit CIDR\" statements in a string delimited by \"\\n\"  # noqa: E501

        :return: The access_list of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :rtype: str
        """
        return self._access_list

    @access_list.setter
    def access_list(self, access_list):
        """Sets the access_list of this UpdateBGPPeerConnectionRequest.

        List of \"in permit CIDR\" and/or \"out permit CIDR\" statements in a string delimited by \"\\n\"  # noqa: E501

        :param access_list: The access_list of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :type: str
        """

        self._access_list = access_list

    @property
    def bgp_password(self):
        """Gets the bgp_password of this UpdateBGPPeerConnectionRequest.  # noqa: E501

        String to be agreed upon by both peers as a simple password.  # noqa: E501

        :return: The bgp_password of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :rtype: str
        """
        return self._bgp_password

    @bgp_password.setter
    def bgp_password(self, bgp_password):
        """Sets the bgp_password of this UpdateBGPPeerConnectionRequest.

        String to be agreed upon by both peers as a simple password.  # noqa: E501

        :param bgp_password: The bgp_password of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :type: str
        """

        self._bgp_password = bgp_password

    @property
    def add_network_distance(self):
        """Gets the add_network_distance of this UpdateBGPPeerConnectionRequest.  # noqa: E501

        Enable network distance for BGP peer  # noqa: E501

        :return: The add_network_distance of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :rtype: bool
        """
        return self._add_network_distance

    @add_network_distance.setter
    def add_network_distance(self, add_network_distance):
        """Sets the add_network_distance of this UpdateBGPPeerConnectionRequest.

        Enable network distance for BGP peer  # noqa: E501

        :param add_network_distance: The add_network_distance of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :type: bool
        """

        self._add_network_distance = add_network_distance

    @property
    def add_network_distance_direction(self):
        """Gets the add_network_distance_direction of this UpdateBGPPeerConnectionRequest.  # noqa: E501

        Add distance direction for BGP peer  # noqa: E501

        :return: The add_network_distance_direction of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :rtype: str
        """
        return self._add_network_distance_direction

    @add_network_distance_direction.setter
    def add_network_distance_direction(self, add_network_distance_direction):
        """Sets the add_network_distance_direction of this UpdateBGPPeerConnectionRequest.

        Add distance direction for BGP peer  # noqa: E501

        :param add_network_distance_direction: The add_network_distance_direction of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :type: str
        """

        self._add_network_distance_direction = add_network_distance_direction

    @property
    def add_network_distance_hops(self):
        """Gets the add_network_distance_hops of this UpdateBGPPeerConnectionRequest.  # noqa: E501

        Distance metric weight indicating distance in hops for BGP peer  # noqa: E501

        :return: The add_network_distance_hops of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :rtype: int
        """
        return self._add_network_distance_hops

    @add_network_distance_hops.setter
    def add_network_distance_hops(self, add_network_distance_hops):
        """Sets the add_network_distance_hops of this UpdateBGPPeerConnectionRequest.

        Distance metric weight indicating distance in hops for BGP peer  # noqa: E501

        :param add_network_distance_hops: The add_network_distance_hops of this UpdateBGPPeerConnectionRequest.  # noqa: E501
        :type: int
        """

        self._add_network_distance_hops = add_network_distance_hops

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UpdateBGPPeerConnectionRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SystemInterface(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "int",
        "name": "str",
        "interface_type": "str",
        "description": "str",
        "ip_internal": "str",
        "mtu": "int",
        "enabled": "bool",
        "status": "str",
        "mask_bits": "str",
        "gateway": "str",
        "system_default": "bool",
        "ip_external": "str",
        "tags": "list[str]",
    }

    attribute_map = {
        "id": "id",
        "name": "name",
        "interface_type": "interface_type",
        "description": "description",
        "ip_internal": "ip_internal",
        "mtu": "mtu",
        "enabled": "enabled",
        "status": "status",
        "mask_bits": "mask_bits",
        "gateway": "gateway",
        "system_default": "system_default",
        "ip_external": "ip_external",
        "tags": "tags",
    }

    def __init__(
        self,
        id=None,
        name=None,
        interface_type=None,
        description=None,
        ip_internal=None,
        mtu=None,
        enabled=None,
        status=None,
        mask_bits=None,
        gateway=None,
        system_default=None,
        ip_external=None,
        tags=None,
    ):  # noqa: E501
        """SystemInterface - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._name = None
        self._interface_type = None
        self._description = None
        self._ip_internal = None
        self._mtu = None
        self._enabled = None
        self._status = None
        self._mask_bits = None
        self._gateway = None
        self._system_default = None
        self._ip_external = None
        self._tags = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if interface_type is not None:
            self.interface_type = interface_type
        if description is not None:
            self.description = description
        if ip_internal is not None:
            self.ip_internal = ip_internal
        if mtu is not None:
            self.mtu = mtu
        if enabled is not None:
            self.enabled = enabled
        if status is not None:
            self.status = status
        if mask_bits is not None:
            self.mask_bits = mask_bits
        self.gateway = gateway
        if system_default is not None:
            self.system_default = system_default
        self.ip_external = ip_external
        if tags is not None:
            self.tags = tags

    @property
    def id(self):
        """Gets the id of this SystemInterface.  # noqa: E501


        :return: The id of this SystemInterface.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SystemInterface.


        :param id: The id of this SystemInterface.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this SystemInterface.  # noqa: E501


        :return: The name of this SystemInterface.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SystemInterface.


        :param name: The name of this SystemInterface.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def interface_type(self):
        """Gets the interface_type of this SystemInterface.  # noqa: E501

        system or system_virtual  # noqa: E501

        :return: The interface_type of this SystemInterface.  # noqa: E501
        :rtype: str
        """
        return self._interface_type

    @interface_type.setter
    def interface_type(self, interface_type):
        """Sets the interface_type of this SystemInterface.

        system or system_virtual  # noqa: E501

        :param interface_type: The interface_type of this SystemInterface.  # noqa: E501
        :type: str
        """

        self._interface_type = interface_type

    @property
    def description(self):
        """Gets the description of this SystemInterface.  # noqa: E501


        :return: The description of this SystemInterface.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this SystemInterface.


        :param description: The description of this SystemInterface.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def ip_internal(self):
        """Gets the ip_internal of this SystemInterface.  # noqa: E501


        :return: The ip_internal of this SystemInterface.  # noqa: E501
        :rtype: str
        """
        return self._ip_internal

    @ip_internal.setter
    def ip_internal(self, ip_internal):
        """Sets the ip_internal of this SystemInterface.


        :param ip_internal: The ip_internal of this SystemInterface.  # noqa: E501
        :type: str
        """

        self._ip_internal = ip_internal

    @property
    def mtu(self):
        """Gets the mtu of this SystemInterface.  # noqa: E501


        :return: The mtu of this SystemInterface.  # noqa: E501
        :rtype: int
        """
        return self._mtu

    @mtu.setter
    def mtu(self, mtu):
        """Sets the mtu of this SystemInterface.


        :param mtu: The mtu of this SystemInterface.  # noqa: E501
        :type: int
        """

        self._mtu = mtu

    @property
    def enabled(self):
        """Gets the enabled of this SystemInterface.  # noqa: E501


        :return: The enabled of this SystemInterface.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this SystemInterface.


        :param enabled: The enabled of this SystemInterface.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    @property
    def status(self):
        """Gets the status of this SystemInterface.  # noqa: E501

        Availability of interface, Up or Down  # noqa: E501

        :return: The status of this SystemInterface.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this SystemInterface.

        Availability of interface, Up or Down  # noqa: E501

        :param status: The status of this SystemInterface.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def mask_bits(self):
        """Gets the mask_bits of this SystemInterface.  # noqa: E501


        :return: The mask_bits of this SystemInterface.  # noqa: E501
        :rtype: str
        """
        return self._mask_bits

    @mask_bits.setter
    def mask_bits(self, mask_bits):
        """Sets the mask_bits of this SystemInterface.


        :param mask_bits: The mask_bits of this SystemInterface.  # noqa: E501
        :type: str
        """

        self._mask_bits = mask_bits

    @property
    def gateway(self):
        """Gets the gateway of this SystemInterface.  # noqa: E501


        :return: The gateway of this SystemInterface.  # noqa: E501
        :rtype: str
        """
        return self._gateway

    @gateway.setter
    def gateway(self, gateway):
        """Sets the gateway of this SystemInterface.


        :param gateway: The gateway of this SystemInterface.  # noqa: E501
        :type: str
        """

        self._gateway = gateway

    @property
    def system_default(self):
        """Gets the system_default of this SystemInterface.  # noqa: E501


        :return: The system_default of this SystemInterface.  # noqa: E501
        :rtype: bool
        """
        return self._system_default

    @system_default.setter
    def system_default(self, system_default):
        """Sets the system_default of this SystemInterface.


        :param system_default: The system_default of this SystemInterface.  # noqa: E501
        :type: bool
        """

        self._system_default = system_default

    @property
    def ip_external(self):
        """Gets the ip_external of this SystemInterface.  # noqa: E501


        :return: The ip_external of this SystemInterface.  # noqa: E501
        :rtype: str
        """
        return self._ip_external

    @ip_external.setter
    def ip_external(self, ip_external):
        """Sets the ip_external of this SystemInterface.


        :param ip_external: The ip_external of this SystemInterface.  # noqa: E501
        :type: str
        """

        self._ip_external = ip_external

    @property
    def tags(self):
        """Gets the tags of this SystemInterface.  # noqa: E501


        :return: The tags of this SystemInterface.  # noqa: E501
        :rtype: list[str]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this SystemInterface.


        :param tags: The tags of this SystemInterface.  # noqa: E501
        :type: list[str]
        """

        self._tags = tags

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SystemInterface):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
